@@ -477,10 +477,7 @@ class FluxAppManager:
         extras = []
 
         fake_root = managed_object.root()
-        # just make this a property
-        # if app_mode == AppMode.FILESERVER:
-        #     root_path = managed_object.remote_workdir
-        # else:
+
         root_path = "/"
         for remote_name in remote_hashes:
             remote_name = Path(remote_name)
@@ -542,11 +539,11 @@ class FluxAppManager:
         local_hashes: dict[str, int],
         remote_hashes: dict[str, int],
     ) -> tuple[list[Path], list[Path]]:
-        print("before")
+
         candidates, missing, modified = self.get_missing_or_modified_objects(
             self.app.app_mode, managed_object, local_hashes, remote_hashes
         )
-        print("after get missing")
+
         extra_objects, unknown = self.get_extra_objects(
             self.app.app_mode, managed_object, local_hashes, remote_hashes
         )
@@ -657,14 +654,11 @@ class FluxAppManager:
             name=managed_object.name
         )
 
-        print("about to resolve deltas")
         # these are in remote absolute form
         object_fragments, objects_to_remove = self.resolve_object_deltas(
             managed_object, local_hashes, remote_hashes
         )
 
-        print("have resolved deltas")
-
         if (
             managed_object.remit.sync_strategy == SyncStrategy.STRICT
             and objects_to_remove
@@ -695,6 +689,11 @@ class FluxAppManager:
     @manage_transport
     async def load_manifest(self, agent: RPCClient):
         """This is solely for the fileserver"""
+
+        if not agent.transport.auth_provider:
+            log.warn("Agent not using auth, unable to sign manifest... skipping")
+            return
+
         component = self.app.get_component(agent.id[2])
         managed_object = component.state_manager.get_object_by_remote_path(WWW_ROOT)
         fileserver_hash = managed_object.local_crc
@@ -778,11 +777,9 @@ class FluxAppManager:
 
             if not managed_object.in_sync and managed_object.concrete_fs.storable:
                 # so here we need to figure any children dirs and set them manually to in_sync
-                print("Resolving dir state")
                 fixed = await self.resolve_directory_state(
                     component, managed_object, agent_proxy
                 )
-                print("dir state resolved")
                 fixed_objects.extend(fixed)
 
             if not managed_object.in_sync and not managed_object.concrete_fs.storable:
